;;;; setup-helper.scm -*- Hen -*-
;;;; Kon Lovett, Mar '09

;;; Release 4 Only!
(use extras)            ;for sprintf
(use data-structures)   ;for ->string
(use files)             ;for make-pathname
(use setup-api)         ;for required-chicken-version

;;; Extension Information

(define (verify-extension-name nam)
  (let ((extnam (->string nam)))
    (unless (string=? extnam (extension-name))
      (error "unexpected extension-name" extnam (extension-name)) ) ) )

;;; Support

;; Filename Support

(define (installation-chicken-home)
  (if (not (installation-prefix)) (chicken-home)
    (make-pathname (installation-prefix) "share/chicken") ) )

(define (installation-repository-path)
  (make-pathname
    (installation-prefix)
    (sprintf "lib/chicken/~a" (##sys#fudge 42))) )

(define (filename bn #!optional en)
  (make-pathname #f (->string bn) (and en (->string en))) )

(define (make-directory dir)
  (cond
    ((string? dir)  dir)
    ((symbol? dir)  (symbol->string dir))
    ((pair? dir)
      (let ((len (length dir)))
        (if (= 1 len) (->string (car dir))
            (make-pathname
              (map ->string (take dir (sub1 len)))
              (->string (last dir))) ) ) )
    (else
      (warning 'make-directory "unknown argument" dir) ) ) )

(define (document-filename bn) (filename bn "html"))

(define (source-filename bn) (filename bn "scm"))

(define (shared-library-filename bn)
  (filename bn ##sys#load-library-extension) )

(define (shared-filename bn)
  (filename bn ##sys#load-dynamic-extension) )

(define (static-library-filename bn) (filename bn "a"))

(define (static-filename bn) (filename bn "o"))

(define (import-filename bn) (filename bn "import"))

(define (source-import-filename bn)
  (source-filename (import-filename bn)) )

(define (shared-import-filename bn)
  (shared-filename (import-filename bn)) )

(define (inline-filename bn) (filename bn "inline"))

(define (program-filename bn)
 (filename bn (and (eq? 'windows (software-type)) "exe")) )

(define (make-repository-pathname bn)
  (make-pathname (installation-repository-path) bn) )

;; File Support

(define (copy-file-to-directory fn dn)
  (copy-file (list fn fn) dn #t dn) )

(define (copy-to-repository fn)
  (copy-file-to-directory fn (installation-repository-path)) )

(define (copy-to-home fn)
  (copy-file-to-directory fn (installation-chicken-home)) )

;; Single File Install Support

(define (install-in-repository fn)
  (when (setup-install-mode)
    (copy-to-repository fn) ) )

(define (install-in-home fn)
  (when (setup-install-mode)
    (copy-to-home fn) ) )

;; SRFI-29 Bundle Support

(define install-srfi-29-bundle)
(let ((*srfi-29-bundles-directory*
        (make-repository-pathname "srfi-29-bundles")))
  (define (srfi-29-bundle-directory spec)
    (if (null? spec) *srfi-29-bundles-directory*
        (make-directory (append (list *srfi-29-bundles-directory*) spec)) ) )
  (set! install-srfi-29-bundle
    (lambda (nam . spec)
      (when (setup-install-mode)
        (unless (directory? *srfi-29-bundles-directory*)
          (error "missing SRFI-29 bundles directory; please install SRFI-29") )
        (let* ((spec (map ->string spec))
               (nam (->string nam))
               (dir (srfi-29-bundle-directory spec)) )
          (copy-file (make-pathname (append '(".") spec) nam)
                     (make-pathname dir nam)
                     #t) ) ) ) ) )

;; Compile Support

(define cc-path-options
  (cond-expand
    (macosx    '(-I/opt/local/include -L/opt/local/lib -I/sw/include -L/sw/lib))
    (else      '())))

; Bad idea to make `-local' the default for a module compile (but not an import compile)
; since it means something like `fluid-let' cannot be used on an exported binding.

(define default-static-compile-options
  (make-parameter '(
    -c
    -optimize-leaf-routines -inline)))

(define default-shared-compile-options
  (make-parameter '(
    -shared
    -optimize-leaf-routines -inline)))

(define default-import-compile-options
  (make-parameter '(
    -shared
    -optimize-leaf-routines -inline -local
    -no-trace -no-lambda-info)))

(define (compile-static nam #!key (options '()) inline?)
  (compile
    ,(source-filename nam)
    ,@(default-static-compile-options)
    -unit ,nam
    ,@(if (memq '-output-file options) '() `(-output-file ,(static-filename nam)))
    ,@(if inline? `(-emit-inline-file ,(inline-filename nam)) '())
    ,@options) )

(define (compile-shared nam #!key (options '()) inline?)
  (compile
    ,(source-filename nam)
    ,@(default-shared-compile-options)
    ,@(if (memq '-output-file options) '() `(-output-file ,(shared-filename nam)))
    ,@(if inline? `(-emit-inline-file ,(inline-filename nam)) '())
    ,@options) )

(define (compile-shared-module nam #!key (options '()) inline?)
  (compile
    ,(source-filename nam)
    ,@(default-shared-compile-options)
    ,@(if (memq '-output-file options) '() `(-output-file ,(shared-filename nam)))
    -emit-import-library ,nam
    ,@(if inline? `(-emit-inline-file ,(inline-filename nam)) '())
    ,@options)
  (compile
    ,(source-import-filename nam)
    ,@(default-import-compile-options)
    -output-file ,(shared-import-filename nam)) )

;; Install Support

(define default-static-install-options
  (make-parameter '()))

(define default-shared-install-options
  (make-parameter '()))

(define default-shared-module-install-options
  (make-parameter '()))

(define default-shared+static-module-install-options
  (make-parameter '()))

(define (install-static-extension nam ver #!key (options '()) (files '()) output-file?)
  (install-extension nam
   `(,@(if output-file? '() `(,(static-filename nam))) ,@files)
   `(,@(default-static-install-options)
     (version ,ver)
     (static ,(static-filename nam))
     (documentation ,(document-filename nam))
     ,@options)) )

(define (install-shared-extension nam ver #!key (options '()) (files '()) output-file?)
  (install-extension nam
   `(,@(if output-file? '() `(,(shared-filename nam))) ,@files)
   `(,@(default-shared-install-options)
     (version ,ver)
     (documentation ,(document-filename nam))
     ,@options)) )

(define (install-shared-extension-module nam ver #!key (options '()) (files '()) output-file?)
  (install-extension nam
   `(,@(if output-file? '() `(,(shared-filename nam)))
     ,(shared-import-filename nam)
     ,@files)
   `(,@(default-shared-module-install-options)
     (version ,ver)
     (documentation ,(document-filename nam))
     ,@options)) )

(define (install-shared+static-extension-module nam ver
          #!key (options '()) (files '()) shared-output-file? static-output-file?)
  (install-extension nam
   `(,@(if shared-output-file? '() `(,(shared-filename nam)))
     ,(shared-import-filename nam)
     ,@(if static-output-file? '() `((static-filename nam)))
     ,@files)
   `(,@(default-shared+static-module-install-options)
     (version ,ver)
     (static ,(static-filename nam))
     (documentation ,(document-filename nam))
     ,@options)) )

;; Setup Support

(define (setup-static-extension nam ver
          #!key (compile-options '()) inline? (install-options '()) (files '()))
  (and-let* ((of (memq '-output-file compile-options)))
    (set! files (append files (list (cadr of)))) )
  (compile-static nam options: compile-options inline?: inline?)
  (install-static-extension nam ver options: install-options files: files) )

(define (setup-shared-extension nam ver
          #!key (compile-options '()) inline? (install-options '()) (files '()))
  (and-let* ((of (memq '-output-file compile-options)))
    (set! files (append files (list (cadr of)))) )
  (compile-shared nam options: compile-options inline?: inline?)
  (install-shared-extension nam ver options: install-options files: files) )

(define (setup-shared-extension-module nam ver
          #!key (compile-options '()) inline? (install-options '()) (files '()))
  (and-let* ((of (memq '-output-file compile-options)))
    (set! files (append files (list (cadr of)))) )
  (compile-shared-module nam options: compile-options inline?: inline?)
  (install-shared-extension-module nam ver options: install-options files: files) )

(define (setup-shared+static-extension-module nam ver
          #!key (shared-compile-options '()) shared-inline?
                (static-compile-options '()) static-inline?
                (install-options '()) (files '()))
  (compile-static nam options: static-compile-options inline?: static-inline?)
  (compile-shared-module nam options: shared-compile-options inline?: shared-inline?)
  (install-shared+static-extension-module nam ver options: install-options files: files) )

;; Empty "Conglomerate" Extension Support

(define (install-extension-tag nam ver) (install-extension nam '() `((version ,ver))))
